@@ -3,7 +3,9 @@ import ctypes.util
 import platform
 
 from ctypes import (
+    cast,
     byref,
+    c_char,
     c_byte,
     c_int,
     c_uint,
@@ -46,6 +48,8 @@ def _find_library():
     # try searching
     if not library_path:
         library_path = ctypes.util.find_library("libsecp256k1")
+    if not library_path:
+        library_path = ctypes.util.find_library("secp256k1")
     # library search failed
     if not library_path:
         if platform.system() == "Linux" and os.path.isfile(
@@ -228,6 +232,19 @@ def _init(flags=(CONTEXT_SIGN | CONTEXT_VERIFY)):
         secp256k1.secp256k1_pedersen_commit.argtypes = [c_void_p, c_char_p, c_char_p, c_uint64, c_char_p]
         secp256k1.secp256k1_pedersen_commit.restype = c_int
 
+        secp256k1.secp256k1_pedersen_blind_generator_blind_sum.argtypes = [
+            c_void_p, # const secp256k1_context* ctx,
+            POINTER(c_uint64), # const uint64_t *value,
+            c_void_p, # const unsigned char* const* generator_blind,
+            c_void_p, # unsigned char* const* blinding_factor,
+            c_size_t, # size_t n_total,
+            c_size_t, # size_t n_inputs
+        ]
+        secp256k1.secp256k1_pedersen_blind_generator_blind_sum.restype = c_int
+
+        secp256k1.secp256k1_pedersen_verify_tally.argtypes = [c_void_p, c_void_p, c_size_t, c_void_p, c_size_t]
+        secp256k1.secp256k1_pedersen_verify_tally.restype = c_int
+
         # rangeproof
         secp256k1.secp256k1_rangeproof_rewind.argtypes = [c_void_p, c_char_p, POINTER(c_uint64), c_char_p, POINTER(c_uint64),
                                                           c_char_p, POINTER(c_uint64), POINTER(c_uint64),
@@ -256,12 +273,64 @@ def _init(flags=(CONTEXT_SIGN | CONTEXT_VERIFY)):
         secp256k1.secp256k1_rangeproof_sign.restype = c_int
 
         # musig
-
         secp256k1.secp256k1_xonly_pubkey_from_pubkey.argtypes = [c_void_p, c_char_p, POINTER(c_int), c_char_p]
         secp256k1.secp256k1_xonly_pubkey_from_pubkey.restype = c_int
 
         secp256k1.secp256k1_musig_pubkey_combine.argtypes = [c_void_p, c_void_p, c_char_p, c_void_p, c_void_p, c_size_t]
         secp256k1.secp256k1_musig_pubkey_combine.restype = c_int
+
+        # surjection proofs
+        secp256k1.secp256k1_surjectionproof_initialize.argtypes = [
+            c_void_p, # const secp256k1_context* ctx,
+            c_char_p, # secp256k1_surjectionproof* proof,
+            POINTER(c_size_t), # size_t *input_index,
+            c_void_p, # c_char_p, # const secp256k1_fixed_asset_tag* fixed_input_tags,
+            c_size_t, # const size_t n_input_tags,
+            c_size_t, # const size_t n_input_tags_to_use,
+            c_char_p, # const secp256k1_fixed_asset_tag* fixed_output_tag,
+            c_size_t, # const size_t n_max_iterations,
+            c_char_p, # const unsigned char *random_seed32
+        ]
+        secp256k1.secp256k1_surjectionproof_initialize.restype = c_int
+
+        secp256k1.secp256k1_surjectionproof_generate.argtypes = [
+            c_void_p, # const secp256k1_context* ctx,
+            c_char_p, # secp256k1_surjectionproof* proof,
+            c_char_p, # const secp256k1_generator* ephemeral_input_tags,
+            c_size_t, # size_t n_ephemeral_input_tags,
+            c_char_p, # const secp256k1_generator* ephemeral_output_tag,
+            c_size_t, # size_t input_index,
+            c_char_p, # const unsigned char *input_blinding_key,
+            c_char_p, # const unsigned char *output_blinding_key
+        ]
+        secp256k1.secp256k1_surjectionproof_generate.restype = c_int
+
+        secp256k1.secp256k1_surjectionproof_verify.argtypes = [
+            c_void_p, # const secp256k1_context* ctx,
+            c_char_p, # const secp256k1_surjectionproof* proof,
+            c_char_p, # const secp256k1_generator* ephemeral_input_tags,
+            c_size_t, # size_t n_ephemeral_input_tags,
+            c_char_p, # const secp256k1_generator* ephemeral_output_tag
+        ]
+        secp256k1.secp256k1_surjectionproof_verify.restype = c_int
+
+        secp256k1.secp256k1_surjectionproof_serialize.argtypes = [
+            c_void_p, # const secp256k1_context* ctx,
+            c_char_p, # unsigned char *output,
+            POINTER(c_size_t), # size_t *outputlen,
+            c_char_p, # const secp256k1_surjectionproof *proof
+        ]
+        secp256k1.secp256k1_surjectionproof_serialize.restype = c_int
+
+        secp256k1.secp256k1_surjectionproof_serialized_size.argtypes = [
+            c_void_p, # const secp256k1_context* ctx,
+            c_char_p, # const secp256k1_surjectionproof* proof
+        ]
+        secp256k1.secp256k1_surjectionproof_serialized_size.restype = c_size_t
+
+        secp256k1.secp256k1_surjectionproof_parse.argtypes = [c_void_p, c_char_p, c_char_p, c_size_t]
+        secp256k1.secp256k1_surjectionproof_parse.restype = c_int
+
     except:
         pass
 
@@ -555,13 +624,34 @@ def pedersen_commit(vbf, value, gen, context=_secp.ctx):
     if len(gen)!=64:
         raise ValueError("Generator should be 64 bytes long")
     if len(vbf)!=32:
-        raise ValueError("Blinding factor should be 32 bytes long")
+        raise ValueError(f"Blinding factor should be 32 bytes long, not {len(vbf)}")
     commit = bytes(64)
     r = _secp.secp256k1_pedersen_commit(context, commit, vbf, value, gen)
     if r == 0:
         raise ValueError("Failed to create commitment")
     return commit
 
+def pedersen_blind_generator_blind_sum(values, gens, vbfs, num_inputs, context=_secp.ctx):
+    vals = (c_uint64 * len(values))(*values)
+    vbf = bytes(vbfs[-1])
+    p = c_char_p(vbf) # obtain a pointer of various types
+    address = cast(p,c_void_p).value
+
+    vbfs_joined = (c_char_p * len(vbfs))(*vbfs[:-1], address)
+    gens_joined = (c_char_p * len(gens))(*gens)
+    res = _secp.secp256k1_pedersen_blind_generator_blind_sum(context, vals, gens_joined, vbfs_joined, len(values), num_inputs)
+    if res == 0:
+        raise ValueError("Failed to get the last blinding factor.")
+    res = (c_char * 32).from_address(address).raw
+    assert len(res) == 32
+    return res
+
+def pedersen_verify_tally(ins, outs, context=_secp.ctx):
+    in_ptr = (c_char_p * len(ins))(*ins)
+    out_ptr = (c_char_p * len(outs))(*outs)
+    res = _secp.secp256k1_pedersen_verify_tally(context, in_ptr, len(in_ptr), out_ptr, len(out_ptr))
+    return bool(res)
+
 # generator
 def generator_parse(inp, context=_secp.ctx):
     if len(inp)!=33:
@@ -665,3 +755,45 @@ def musig_pubkey_combine(*args, context=_secp.ctx):
     if res == 0:
         raise ValueError("Failed to combine pubkeys")
     return pub
+
+# surjection proof
+def surjectionproof_initialize(in_tags, out_tag, tags_to_use=None, iterations=100, seed=None, context=_secp.ctx):
+    if tags_to_use is None:
+        tags_to_use = min(3, len(in_tags))
+    if seed is None:
+        seed = os.urandom(32)
+    proof = bytes(4+8+256//8+32*257)
+    pointer = POINTER(c_size_t)
+    input_index = pointer(c_size_t(0))
+    input_tags = b"".join(in_tags)
+    res = _secp.secp256k1_surjectionproof_initialize(context, proof, input_index, input_tags, len(in_tags), tags_to_use, out_tag, iterations, seed)
+    if res == 0:
+        raise RuntimeError("Failed to initialize the proof")
+    return proof, input_index.contents.value
+
+def surjectionproof_generate(proof, in_idx, in_tags, out_tag, in_abf, out_abf, context=_secp.ctx):
+    res = _secp.secp256k1_surjectionproof_generate(context, proof, b"".join(in_tags), len(in_tags), out_tag, in_idx, in_abf, out_abf)
+    if not res:
+        raise RuntimeError("Failed to generate surjection proof")
+    return proof
+
+def surjectionproof_verify(proof, in_tags, out_tag, context=_secp.ctx):
+    res = _secp.secp256k1_surjectionproof_verify(context, proof, b"".join(in_tags), len(in_tags), out_tag)
+    return bool(res)
+
+def surjectionproof_serialize(proof, context=_secp.ctx):
+    s = _secp.secp256k1_surjectionproof_serialized_size(context, proof)
+    b = bytes(s)
+    pointer = POINTER(c_size_t)
+    sz = pointer(c_size_t(s))
+    _secp.secp256k1_surjectionproof_serialize(context, b, sz, proof)
+    if s != sz.contents.value:
+        raise RuntimeError("Failed to serialize surjection proof - size mismatch")
+    return b
+
+def surjectionproof_parse(proof, context=_secp.ctx):
+    parsed_proof = bytes(4+8+256//8+32*257)
+    res = _secp.secp256k1_surjectionproof_parse(context, parsed_proof, proof, len(proof))
+    if res == 0:
+        raise RuntimeError("Failed to parse surjection proof")
+    return parsed_proof
