@@ -5,6 +5,7 @@ This is a fallback option if the library can't do ctypes bindings to secp256k1 l
 import random
 import hmac
 
+
 def modinv(a, n):
     """Compute the modular inverse of a modulo n using the extended Euclidean
     Algorithm. See https://en.wikipedia.org/wiki/Extended_Euclidean_algorithm#Modular_integers.
@@ -22,9 +23,11 @@ def modinv(a, n):
         t1 += n
     return t1
 
+
 def xor_bytes(b0, b1):
     return bytes(x ^ y for (x, y) in zip(b0, b1))
 
+
 def jacobi_symbol(n, k):
     """Compute the Jacobi symbol of n modulo k
 
@@ -38,14 +41,15 @@ def jacobi_symbol(n, k):
         while n & 1 == 0:
             n >>= 1
             r = k & 7
-            t ^= (r == 3 or r == 5)
+            t ^= r == 3 or r == 5
         n, k = k, n
-        t ^= (n & k & 3 == 3)
+        t ^= n & k & 3 == 3
         n = n % k
     if k == 1:
         return -1 if t else 1
     return 0
 
+
 def modsqrt(a, p):
     """Compute the square root of a modulo p when p % 4 = 3.
 
@@ -59,11 +63,12 @@ def modsqrt(a, p):
     """
     if p % 4 != 3:
         raise NotImplementedError("modsqrt only implemented for p % 4 = 3")
-    sqrt = pow(a, (p + 1)//4, p)
+    sqrt = pow(a, (p + 1) // 4, p)
     if pow(sqrt, 2, p) == a % p:
         return sqrt
     return None
 
+
 class EllipticCurve:
     def __init__(self, p, a, b):
         """Initialize elliptic curve y^2 = x^3 + a*x + b over GF(p)."""
@@ -79,7 +84,7 @@ class EllipticCurve:
         if z1 == 0:
             return None
         inv = modinv(z1, self.p)
-        inv_2 = (inv**2) % self.p
+        inv_2 = (inv ** 2) % self.p
         inv_3 = (inv_2 * inv) % self.p
         return ((inv_2 * x1) % self.p, (inv_3 * y1) % self.p, 1)
 
@@ -97,7 +102,17 @@ class EllipticCurve:
         x1, y1, z1 = p1
         z2 = pow(z1, 2, self.p)
         z4 = pow(z2, 2, self.p)
-        return z1 != 0 and (pow(x1, 3, self.p) + self.a * x1 * z4 + self.b * z2 * z4 - pow(y1, 2, self.p)) % self.p == 0
+        return (
+            z1 != 0
+            and (
+                pow(x1, 3, self.p)
+                + self.a * x1 * z4
+                + self.b * z2 * z4
+                - pow(y1, 2, self.p)
+            )
+            % self.p
+            == 0
+        )
 
     def is_x_coord(self, x):
         """Test whether x is a valid X coordinate on the curve."""
@@ -120,17 +135,17 @@ class EllipticCurve:
         x1, y1, z1 = p1
         if z1 == 0:
             return (0, 1, 0)
-        y1_2 = (y1**2) % self.p
-        y1_4 = (y1_2**2) % self.p
-        x1_2 = (x1**2) % self.p
-        s = (4*x1*y1_2) % self.p
-        m = 3*x1_2
+        y1_2 = (y1 ** 2) % self.p
+        y1_4 = (y1_2 ** 2) % self.p
+        x1_2 = (x1 ** 2) % self.p
+        s = (4 * x1 * y1_2) % self.p
+        m = 3 * x1_2
         if self.a:
             m += self.a * pow(z1, 4, self.p)
         m = m % self.p
-        x2 = (m**2 - 2*s) % self.p
-        y2 = (m*(s - x2) - 8*y1_4) % self.p
-        z2 = (2*y1*z1) % self.p
+        x2 = (m ** 2 - 2 * s) % self.p
+        y2 = (m * (s - x2) - 8 * y1_4) % self.p
+        z2 = (2 * y1 * z1) % self.p
         return (x2, y2, z2)
 
     def add_mixed(self, p1, p2):
@@ -139,28 +154,28 @@ class EllipticCurve:
         See https://en.wikibooks.org/wiki/Cryptography/Prime_Curve/Jacobian_Coordinates - Point Addition (with affine point)"""
         x1, y1, z1 = p1
         x2, y2, z2 = p2
-        assert(z2 == 1)
+        assert z2 == 1
         # Adding to the point at infinity is a no-op
         if z1 == 0:
             return p2
-        z1_2 = (z1**2) % self.p
+        z1_2 = (z1 ** 2) % self.p
         z1_3 = (z1_2 * z1) % self.p
         u2 = (x2 * z1_2) % self.p
         s2 = (y2 * z1_3) % self.p
         if x1 == u2:
-            if (y1 != s2):
+            if y1 != s2:
                 # p1 and p2 are inverses. Return the point at infinity.
                 return (0, 1, 0)
             # p1 == p2. The formulas below fail when the two points are equal.
             return self.double(p1)
         h = u2 - x1
         r = s2 - y1
-        h_2 = (h**2) % self.p
+        h_2 = (h ** 2) % self.p
         h_3 = (h_2 * h) % self.p
         u1_h_2 = (x1 * h_2) % self.p
-        x3 = (r**2 - h_3 - 2*u1_h_2) % self.p
-        y3 = (r*(u1_h_2 - x3) - y1*h_3) % self.p
-        z3 = (h*z1) % self.p
+        x3 = (r ** 2 - h_3 - 2 * u1_h_2) % self.p
+        y3 = (r * (u1_h_2 - x3) - y1 * h_3) % self.p
+        z3 = (h * z1) % self.p
         return (x3, y3, z3)
 
     def add(self, p1, p2):
@@ -179,28 +194,28 @@ class EllipticCurve:
             return self.add_mixed(p2, p1)
         if z2 == 1:
             return self.add_mixed(p1, p2)
-        z1_2 = (z1**2) % self.p
+        z1_2 = (z1 ** 2) % self.p
         z1_3 = (z1_2 * z1) % self.p
-        z2_2 = (z2**2) % self.p
+        z2_2 = (z2 ** 2) % self.p
         z2_3 = (z2_2 * z2) % self.p
         u1 = (x1 * z2_2) % self.p
         u2 = (x2 * z1_2) % self.p
         s1 = (y1 * z2_3) % self.p
         s2 = (y2 * z1_3) % self.p
         if u1 == u2:
-            if (s1 != s2):
+            if s1 != s2:
                 # p1 and p2 are inverses. Return the point at infinity.
                 return (0, 1, 0)
             # p1 == p2. The formulas below fail when the two points are equal.
             return self.double(p1)
         h = u2 - u1
         r = s2 - s1
-        h_2 = (h**2) % self.p
+        h_2 = (h ** 2) % self.p
         h_3 = (h_2 * h) % self.p
         u1_h_2 = (u1 * h_2) % self.p
-        x3 = (r**2 - h_3 - 2*u1_h_2) % self.p
-        y3 = (r*(u1_h_2 - x3) - s1*h_3) % self.p
-        z3 = (h*z1*z2) % self.p
+        x3 = (r ** 2 - h_3 - 2 * u1_h_2) % self.p
+        y3 = (r * (u1_h_2 - x3) - s1 * h_3) % self.p
+        z3 = (h * z1 * z2) % self.p
         return (x3, y3, z3)
 
     def mul(self, ps):
@@ -212,17 +227,23 @@ class EllipticCurve:
         for i in range(255, -1, -1):
             r = self.double(r)
             for (p, n) in ps:
-                if ((n >> i) & 1):
+                if (n >> i) & 1:
                     r = self.add(r, p)
         return r
 
-SECP256K1_FIELD_SIZE = 2**256 - 2**32 - 977
+
+SECP256K1_FIELD_SIZE = 2 ** 256 - 2 ** 32 - 977
 SECP256K1 = EllipticCurve(SECP256K1_FIELD_SIZE, 0, 7)
-SECP256K1_G = (0x79BE667EF9DCBBAC55A06295CE870B07029BFCDB2DCE28D959F2815B16F81798, 0x483ADA7726A3C4655DA4FBFC0E1108A8FD17B448A68554199C47D08FFB10D4B8, 1)
+SECP256K1_G = (
+    0x79BE667EF9DCBBAC55A06295CE870B07029BFCDB2DCE28D959F2815B16F81798,
+    0x483ADA7726A3C4655DA4FBFC0E1108A8FD17B448A68554199C47D08FFB10D4B8,
+    1,
+)
 SECP256K1_ORDER = 0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEBAAEDCE6AF48A03BBFD25E8CD0364141
 SECP256K1_ORDER_HALF = SECP256K1_ORDER // 2
 
-class ECPubKey():
+
+class ECPubKey:
     """A secp256k1 public key"""
 
     def __init__(self):
@@ -231,14 +252,18 @@ class ECPubKey():
 
     def set(self, data):
         """Construct a public key from a serialization in compressed or uncompressed format"""
-        if (len(data) == 65 and data[0] == 0x04):
-            p = (int.from_bytes(data[1:33], 'big'), int.from_bytes(data[33:65], 'big'), 1)
+        if len(data) == 65 and data[0] == 0x04:
+            p = (
+                int.from_bytes(data[1:33], "big"),
+                int.from_bytes(data[33:65], "big"),
+                1,
+            )
             self.valid = SECP256K1.on_curve(p)
             if self.valid:
                 self.p = p
                 self.compressed = False
-        elif (len(data) == 33 and (data[0] == 0x02 or data[0] == 0x03)):
-            x = int.from_bytes(data[1:33], 'big')
+        elif len(data) == 33 and (data[0] == 0x02 or data[0] == 0x03):
+            x = int.from_bytes(data[1:33], "big")
             if SECP256K1.is_x_coord(x):
                 p = SECP256K1.lift_x(x)
                 # Make the Y coordinate odd if required (lift_x always produces
@@ -262,76 +287,78 @@ class ECPubKey():
         return self.valid
 
     def get_bytes(self):
-        assert(self.valid)
+        assert self.valid
         p = SECP256K1.affine(self.p)
         if p is None:
             return None
         if self.compressed:
-            return bytes([0x02 + (p[1] & 1)]) + p[0].to_bytes(32, 'big')
+            return bytes([0x02 + (p[1] & 1)]) + p[0].to_bytes(32, "big")
         else:
-            return bytes([0x04]) + p[0].to_bytes(32, 'big') + p[1].to_bytes(32, 'big')
+            return bytes([0x04]) + p[0].to_bytes(32, "big") + p[1].to_bytes(32, "big")
 
     def verify_ecdsa(self, sig, msg, low_s=True):
         """Verify a strictly DER-encoded ECDSA signature against this pubkey.
 
         See https://en.wikipedia.org/wiki/Elliptic_Curve_Digital_Signature_Algorithm for the
         ECDSA verifier algorithm"""
-        assert(self.valid)
+        assert self.valid
 
         # Extract r and s from the DER formatted signature. Return false for
         # any DER encoding errors.
-        if (sig[1] + 2 != len(sig)):
+        if sig[1] + 2 != len(sig):
             return False
-        if (len(sig) < 4):
+        if len(sig) < 4:
             return False
-        if (sig[0] != 0x30):
+        if sig[0] != 0x30:
             return False
-        if (sig[2] != 0x02):
+        if sig[2] != 0x02:
             return False
         rlen = sig[3]
-        if (len(sig) < 6 + rlen):
+        if len(sig) < 6 + rlen:
             return False
         if rlen < 1 or rlen > 33:
             return False
         if sig[4] >= 0x80:
             return False
-        if (rlen > 1 and (sig[4] == 0) and not (sig[5] & 0x80)):
+        if rlen > 1 and (sig[4] == 0) and not (sig[5] & 0x80):
             return False
-        r = int.from_bytes(sig[4:4+rlen], 'big')
-        if (sig[4+rlen] != 0x02):
+        r = int.from_bytes(sig[4 : 4 + rlen], "big")
+        if sig[4 + rlen] != 0x02:
             return False
-        slen = sig[5+rlen]
+        slen = sig[5 + rlen]
         if slen < 1 or slen > 33:
             return False
-        if (len(sig) != 6 + rlen + slen):
+        if len(sig) != 6 + rlen + slen:
             return False
-        if sig[6+rlen] >= 0x80:
+        if sig[6 + rlen] >= 0x80:
             return False
-        if (slen > 1 and (sig[6+rlen] == 0) and not (sig[7+rlen] & 0x80)):
+        if slen > 1 and (sig[6 + rlen] == 0) and not (sig[7 + rlen] & 0x80):
             return False
-        s = int.from_bytes(sig[6+rlen:6+rlen+slen], 'big')
+        s = int.from_bytes(sig[6 + rlen : 6 + rlen + slen], "big")
 
         # Verify that r and s are within the group order
         if r < 1 or s < 1 or r >= SECP256K1_ORDER or s >= SECP256K1_ORDER:
             return False
         if low_s and s >= SECP256K1_ORDER_HALF:
             return False
-        z = int.from_bytes(msg, 'big')
+        z = int.from_bytes(msg, "big")
 
         # Run verifier algorithm on r, s
         w = modinv(s, SECP256K1_ORDER)
-        u1 = z*w % SECP256K1_ORDER
-        u2 = r*w % SECP256K1_ORDER
+        u1 = z * w % SECP256K1_ORDER
+        u2 = r * w % SECP256K1_ORDER
         R = SECP256K1.affine(SECP256K1.mul([(SECP256K1_G, u1), (self.p, u2)]))
         if R is None or (R[0] % SECP256K1_ORDER) != r:
             return False
         return True
 
+
 def generate_privkey():
     """Generate a valid random 32-byte private key."""
-    return random.randrange(1, SECP256K1_ORDER).to_bytes(32, 'big')
+    return random.randrange(1, SECP256K1_ORDER).to_bytes(32, "big")
 
-class ECKey():
+
+class ECKey:
     """A secp256k1 private key"""
 
     def __init__(self):
@@ -339,9 +366,9 @@ class ECKey():
 
     def set(self, secret, compressed):
         """Construct a private key object with given 32-byte secret and compressed flag."""
-        assert(len(secret) == 32)
-        secret = int.from_bytes(secret, 'big')
-        self.valid = (secret > 0 and secret < SECP256K1_ORDER)
+        assert len(secret) == 32
+        secret = int.from_bytes(secret, "big")
+        self.valid = secret > 0 and secret < SECP256K1_ORDER
         if self.valid:
             self.secret = secret
             self.compressed = compressed
@@ -352,8 +379,8 @@ class ECKey():
 
     def get_bytes(self):
         """Retrieve the 32-byte representation of this key."""
-        assert(self.valid)
-        return self.secret.to_bytes(32, 'big')
+        assert self.valid
+        return self.secret.to_bytes(32, "big")
 
     @property
     def is_valid(self):
@@ -365,7 +392,7 @@ class ECKey():
 
     def get_pubkey(self):
         """Compute an ECPubKey object for this secret key."""
-        assert(self.valid)
+        assert self.valid
         ret = ECPubKey()
         p = SECP256K1.mul([(SECP256K1_G, self.secret)])
         ret.p = p
@@ -378,8 +405,8 @@ class ECKey():
 
         See https://en.wikipedia.org/wiki/Elliptic_Curve_Digital_Signature_Algorithm for the
         ECDSA signer algorithm."""
-        assert(self.valid)
-        z = int.from_bytes(msg, 'big')
+        assert self.valid
+        z = int.from_bytes(msg, "big")
         k = deterministic_k(self.secret, z)
         R = SECP256K1.affine(SECP256K1.mul([(SECP256K1_G, k)]))
         r = R[0] % SECP256K1_ORDER
@@ -389,30 +416,38 @@ class ECKey():
         # Represent in DER format. The byte representations of r and s have
         # length rounded up (255 bits becomes 32 bytes and 256 bits becomes 33
         # bytes).
-        rb = r.to_bytes((r.bit_length() + 8) // 8, 'big')
-        sb = s.to_bytes((s.bit_length() + 8) // 8, 'big')
-        return b'\x30' + bytes([4 + len(rb) + len(sb), 2, len(rb)]) + rb + bytes([2, len(sb)]) + sb
+        rb = r.to_bytes((r.bit_length() + 8) // 8, "big")
+        sb = s.to_bytes((s.bit_length() + 8) // 8, "big")
+        return (
+            b"\x30"
+            + bytes([4 + len(rb) + len(sb), 2, len(rb)])
+            + rb
+            + bytes([2, len(sb)])
+            + sb
+        )
+
 
 def deterministic_k(secret, z):
     # RFC6979, optimized for secp256k1
-    k = b'\x00' * 32
-    v = b'\x01' * 32
+    k = b"\x00" * 32
+    v = b"\x01" * 32
     if z > SECP256K1_ORDER:
         z -= SECP256K1_ORDER
-    z_bytes = z.to_bytes(32, 'big')
-    secret_bytes = secret.to_bytes(32, 'big')
-    k = hmac.new(k, v + b'\x00' + secret_bytes + z_bytes, "sha256").digest()
+    z_bytes = z.to_bytes(32, "big")
+    secret_bytes = secret.to_bytes(32, "big")
+    k = hmac.new(k, v + b"\x00" + secret_bytes + z_bytes, "sha256").digest()
     v = hmac.new(k, v, "sha256").digest()
-    k = hmac.new(k, v + b'\x01' + secret_bytes + z_bytes, "sha256").digest()
+    k = hmac.new(k, v + b"\x01" + secret_bytes + z_bytes, "sha256").digest()
     v = hmac.new(k, v, "sha256").digest()
     while True:
         v = hmac.new(k, v, "sha256").digest()
-        candidate = int.from_bytes(v, 'big')
+        candidate = int.from_bytes(v, "big")
         if candidate >= 1 and candidate < SECP256K1_ORDER:
             return candidate
-        k = hmac.new(k, v + b'\x00', "sha256").digest()
+        k = hmac.new(k, v + b"\x00", "sha256").digest()
         v = hmac.new(k, v, "sha256").digest()
 
+
 def compute_xonly_pubkey(key):
     """Compute an x-only (32 byte) public key from a (32 byte) private key.
 
@@ -420,11 +455,12 @@ def compute_xonly_pubkey(key):
     """
 
     assert len(key) == 32
-    x = int.from_bytes(key, 'big')
+    x = int.from_bytes(key, "big")
     if x == 0 or x >= SECP256K1_ORDER:
         return (None, None)
     P = SECP256K1.affine(SECP256K1.mul([(SECP256K1_G, x)]))
-    return (P[0].to_bytes(32, 'big'), not SECP256K1.has_even_y(P))
+    return (P[0].to_bytes(32, "big"), not SECP256K1.has_even_y(P))
+
 
 def tweak_add_privkey(key, tweak):
     """Tweak a private key (after negating it if needed)."""
@@ -432,18 +468,19 @@ def tweak_add_privkey(key, tweak):
     assert len(key) == 32
     assert len(tweak) == 32
 
-    x = int.from_bytes(key, 'big')
+    x = int.from_bytes(key, "big")
     if x == 0 or x >= SECP256K1_ORDER:
         return None
     if not SECP256K1.has_even_y(SECP256K1.mul([(SECP256K1_G, x)])):
-       x = SECP256K1_ORDER - x
-    t = int.from_bytes(tweak, 'big')
+        x = SECP256K1_ORDER - x
+    t = int.from_bytes(tweak, "big")
     if t >= SECP256K1_ORDER:
         return None
     x = (x + t) % SECP256K1_ORDER
     if x == 0:
         return None
-    return x.to_bytes(32, 'big')
+    return x.to_bytes(32, "big")
+
 
 def tweak_add_pubkey(key, tweak):
     """Tweak a public key and return whether the result had to be negated."""
@@ -451,16 +488,16 @@ def tweak_add_pubkey(key, tweak):
     assert len(key) == 32
     assert len(tweak) == 32
 
-    x_coord = int.from_bytes(key, 'big')
+    x_coord = int.from_bytes(key, "big")
     if x_coord >= SECP256K1_FIELD_SIZE:
         return None
     P = SECP256K1.lift_x(x_coord)
     if P is None:
         return None
-    t = int.from_bytes(tweak, 'big')
+    t = int.from_bytes(tweak, "big")
     if t >= SECP256K1_ORDER:
         return None
     Q = SECP256K1.affine(SECP256K1.mul([(SECP256K1_G, t), (P, 1)]))
     if Q is None:
         return None
-    return (Q[0].to_bytes(32, 'big'), not SECP256K1.has_even_y(Q))
+    return (Q[0].to_bytes(32, "big"), not SECP256K1.has_even_y(Q))
